#!/usr/bin/env python3

# Copyright © 2023-2025, Meheret Tesfaye Batu <meherett.batu@gmail.com>
# Distributed under the MIT software license, see the accompanying
# file COPYING or https://opensource.org/license/mit

from typing import List

__name__: str = "bip38"
__version__: str = "v1.4.0"
__license__: str = "MIT"
__author__: str = "Meheret Tesfaye Batu"
__email__: str = "meherett.batu@gmail.com"
__documentation__: str = "https://bip38.readthedocs.com"
__description__: str = "A Python library for implementation of Bitcoin Improvement Proposal - 0038 / BIP38 protocol."
__url__: str = "https://hdwallet.io"
__source__: str = "https://github.com/hdwallet-io/python-bip38"
__changelog__: str = f"{__source__}/blob/master/CHANGELOG.md"
__tracker__: str = f"{__url__}/issues"
__keywords__: List[str] = [
    "bip38", "bitcoin", "private-key", "hdwallet", "encrypt", "decrypt", "passphrase", "wif", "security"
]
__websites__: List[str] = [
    "https://talonlab.org",
    "https://talonlab.gitbook.io/bip38",
    __documentation__,
    "https://hdwallet.online",
    "https://hd.wallet",  # On Web3 domain
    __url__
]